/* This file is part of the Amalthea library.
 *
 * Copyright (C) 2019 Eugene 'Vindex' Stulin
 *
 * The Amalthea library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This libary is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library. If not, see <http://www.gnu.org/licenses/>.
 */

module amalthea.math;

unittest {
    assert(0 == gcd(0, 0));
    assert(gcd(4, 0) == 4 && 4 == gcd(0, 4));
    assert(gcd(1, 1) == 1);
    assert(gcd(1, 0) == 1 && gcd(0, 1) == 1);
    assert(gcd(1, 2) == 1 && gcd(2, 1) == 1);
    assert(gcd(70, 105) == 35 && gcd(105, 70) == 35);
}
/*******************************************************************************
 * Greatest common divisor.
 */
ulong gcd(ulong a, ulong b) {
    if (a == 0) {
        return b;
    }
    if (b == 0) {
        return a;
    }
    while (a != b) {
        if (a > b) {
            a -= b;
        } else {
            b -= a;
        }
    }
    return a;
}


/*******************************************************************************
 * Least common multiple.
 */
ulong lcm(ulong a, ulong b) {
    return a*b / gcd(a, b);
}


/*******************************************************************************
 * This function checks if a number is prime.
 */
bool isPrimeNumber(ulong n) {
    if (n < 2) {
        return false;
    }
    for (ulong i = 2; i <= n/2; ++i) {
        if (n%i == 0) {
            return false;
        }
    }
    return true;
}


/*******************************************************************************
 * Range of Fibonacci numbers.
 */
struct FibonacciRange(T) if (is(T : ulong)) {
    T prevValue = 0;
    T currValue = 1;
    
    @property empty() const {
        return false;
    }
    
    T popFront() {
        currValue += prevValue;
        prevValue = currValue - prevValue;
        return prevValue;
    }
    
    T front() const {
        return currValue;
    }
}
unittest {
    import std.range : take;
    import std.array : array;
    FibonacciRange!int fib;
    auto fibSequence = array(take(fib, 8));
    assert(fibSequence == [1, 1, 2, 3, 5, 8, 13, 21]);
}


/*******************************************************************************
 * Logarithm.
 */
real logarithm(real b, real n) {
    import std.math : log;
    return log(b)/log(n);
}
unittest {
    assert(logarithm(1024*1024, 1024) == 2);
}
